unit GSMCtrFr;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, StdCtrls, GSM, ExtCtrls, Gauges, dm_GSM, Buttons, ComCtrls,
  ToolWin, ImgList, Terminal, ActnList;

const
  glRed = 1;
  glGreen = 3;
  glYellow = 5;
  glNone = 0;

type
  TGSMCentralForm = class(TForm)
    ImageList: TImageList;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    Panel2: TPanel;
    GroupBox2: TGroupBox;
    IndNetRegStatus: TSpeedButton;
    SpeedButton3: TSpeedButton;
    SpeedButton4: TSpeedButton;
    Panel1: TPanel;
    Panel6: TPanel;
    StatusBar: TStatusBar;
    Semafor: TSpeedButton;
    SpeedButton1: TSpeedButton;
    TabSheet5: TTabSheet;
    Panel7: TPanel;
    TabSheet6: TTabSheet;
    TabSheet7: TTabSheet;
    Panel8: TPanel;
    Panel9: TPanel;
    ModemLog: TTerminal;
    SMSLog: TTerminal;
    Label1: TLabel;
    DA: TEdit;
    Label2: TLabel;
    Message: TEdit;
    BitBtn1: TBitBtn;
    AutoShowModemLog: TCheckBox;
    ConfigureGroupbox: TGroupBox;
    SCAEdit: TEdit;
    PinEdit: TEdit;
    PINLabel: TLabel;
    SCALabel: TLabel;
    CommRadioGroup: TRadioGroup;
    ModemComboBox: TComboBox;
    ModemLabel: TLabel;
    SMSLOGCheckBox: TCheckBox;
    GSMLOGCheckBox: TCheckBox;
    SMSLogButton: TButton;
    GSMLogButton: TButton;
    SMSLogSaveDialog: TSaveDialog;
    GSMLogSaveDialog: TSaveDialog;
    Label3: TLabel;
    SMSFormat: TComboBox;
    GroupBox1: TGroupBox;
    CatchSMSMT: TCheckBox;
    CatchSMSCB: TCheckBox;
    CatchSMSDS: TCheckBox;
    OnlyIndication: TCheckBox;
    ManufacturerId: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure SpeedButton3Click(Sender: TObject);
    procedure IndNetRegStatusClick(Sender: TObject);
    procedure StatusBarClick(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ModemLogKeyPress(Sender: TObject; var Key: Char);
    procedure BitBtn1Click(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure SMSLOGCheckBoxClick(Sender: TObject);
    procedure SMSLogButtonClick(Sender: TObject);
    procedure GSMLogButtonClick(Sender: TObject);
    procedure GSMLOGCheckBoxClick(Sender: TObject);
  private
    FSaveActivePage: TTabSheet;
    procedure CheckGSMActive;
    procedure cmNetRegistrationChanged(var Msg: TMessage); message cm_NetRegistrationChanged;
    procedure cmModemStatusChanged(var Msg: TMessage); message cm_ModemStatusChanged;
    procedure SendSMSPhn(const aDA, S: string);
    function GetGlyphNum(aGlyph: Byte; aOnOff: Boolean): Integer;
    procedure StatusLineOnHint(Sender: TObject);
    procedure ShowModemLog;
    procedure HideModemLog;
    procedure GetConfiguration;
    procedure SetConfiguration;
 public
    { Public declarations }
  end;

var
  GSMCentralForm: TGSMCentralForm;

implementation
uses
  AuxGSM, AuxStr, IniFiles, CmdLine;

resourcestring
  sModemNotActive = 'Device is not active';

{$R *.DFM}

const
  SendPrefix = '+:';
  MaxSMSLength = 160;

  CaptionStr = 'GSM Central [%s]';

procedure TGSMCentralForm.FormCreate(Sender: TObject);
var
  i: TGSMEquipment;
begin
  GSMDataModule.ModemLog:= ModemLog;
  GSMDataModule.SMSLog:= SMSLog;
  PageControl1.ActivePage:= TabSheet1;
  Semafor.Align:= alClient;
  Caption:= Format(CaptionStr, [GSMEquipmentLongName[GSMDataModule.GSM.Equipment]]);
  ImageList.GetBitmap(GetGlyphNum(glNone, False), IndNetRegStatus.Glyph);
  ImageList.GetBitmap(GetGlyphNum(glNone, False), Semafor.Glyph);
  for i:=Low(TGSMEquipment) to High(TGSMEquipment) do
    ModemComboBox.Items.Add(GSMEquipmentLongName[i]);
  //  GSMDataModule.Comm1.Open;
  Application.OnHint:= StatusLineOnHint;
end;

function TGSMCentralForm.GetGlyphNum;
begin
  Result:= aGlyph;
  Inc(Result, Byte((aGlyph > 0) and aOnOff));
end;

procedure TGSMCentralForm.CheckGSMActive;
begin
  if not GSMDataModule.Active then
    GSMError(sModemNotActive);
end;

procedure TGSMCentralForm.SendSMSPhn;
var
  aSMS: TMOSMS;
begin
  CheckGSMActive;
  aSMS:= TMOSMS.Create;
  try
    aSMS.GSM:= GSMDataModule.GSM;
    aSMS.DA:= aDA;
    aSMS.UD:= Copy(ISOtoSMS(S),1,MaxSMSLength);
    GSMDataModule.SendSMS(aSMS);
    GSMDataModule.LogSMS(aSMS);
  finally
    aSMS.Free;
  end;
end;

procedure TGSMCentralForm.cmNetRegistrationChanged;
var
  Col: Byte;
begin
  case Msg.wParam of
    1: Col:= glGreen;
    2: Col:= glYellow;
  else
    Col:= glRed;
  end;
  IndNetRegStatus.Glyph:= nil;
  ImageList.GetBitmap(GetGlyphNum(Col, True), IndNetRegStatus.Glyph);
end;

procedure TGSMCentralForm.cmModemStatusChanged;
var
  Col: Byte;
  i: Integer;
begin
  for i:=0 to ConfigureGroupBox.ControlCount-1 do
    ConfigureGroupBox.Controls[i].Enabled := not GSMDataModule.GSM.Active;

  with SMSLogCheckBox do
    SMSLogButton.Enabled := Enabled and Checked;
  with GSMLogCheckBox do
    GSMLogButton.Enabled := Enabled and Checked;

  case Msg.wParam and $7F of
    0: Col:= glRed;
    1:
      begin
        Col:= glGreen;
        with GSMDataModule do
          if (Msg.wParam and $80 <> 0) and GSM.Active then
          begin
            ManufacturerId.Lines.Text:= Format('%s/%s'#13#10'%s'#13#10'S/N: %s', [GSM.ManufacturerId, GSM.ModelId, GSM.RevisionId, GSM.SerialNumberId]);
          end;
      end;
  else
    Col:= glNone;
  end;
  Semafor.Glyph:= nil;
  ImageList.GetBitmap(GetGlyphNum(Col, True), Semafor.Glyph);
  Application.ProcessMessages;
end;

procedure TGSMCentralForm.SpeedButton3Click(Sender: TObject);
var MakeActive: Boolean;
begin
  ShowModemLog;
  try
    MakeActive:=Boolean((Sender as TControl).Tag);
    if MakeActive then
      SetConfiguration;
    GSMDataModule.Active:=MakeActive;
  finally
    HideModemLog;
  end;
end;

procedure TGSMCentralForm.IndNetRegStatusClick(Sender: TObject);
var
  Sg: TStrings;
  I: Integer;
begin
  CheckGSMActive;
  with GSMDataModule do
  begin
    Sg:= TStringList.Create;
    try
      if GSM.SendATCommand('AT+CSQ', atrCode+atrParams+atrATResponse, Sg) = 0 then
      begin
        I:= 1;
        I:= StrToNum(ExtractParam(Sg[0], I));
        if I = 99 then begin
                         IndNetRegStatus.Caption:= '?';
                         I:= 0;
                       end
                  else IndNetRegStatus.Caption:= Format('%d dBm', [2*I + (-113)]);
      end;
    finally
      Sg.Free;
    end;
  end;
end;

procedure TGSMCentralForm.StatusLineOnHint;
begin
  StatusBar.SimpleText:= GetLongHint(Application.Hint);
end;

procedure TGSMCentralForm.StatusBarClick(Sender: TObject);
begin
  with StatusBar do
    SimpleText:= GetShortHint(Hint);
end;

procedure TGSMCentralForm.SpeedButton1Click(Sender: TObject);
begin
  ShowModemLog;
  try
    GSMDataModule.GSM.SwitchOff;
  finally
    HideModemLog;
  end;
end;

procedure TGSMCentralForm.FormShow(Sender: TObject);
begin
  GetConfiguration;
  if IsThereCmd('START', clUpcase) then
    SpeedButton3Click(SpeedButton3);
end;

procedure TGSMCentralForm.ModemLogKeyPress(Sender: TObject; var Key: Char);
begin
  GSMDataModule.GSM.COMDevice.Send(Key);
  Key:= #0;
end;

procedure TGSMCentralForm.BitBtn1Click(Sender: TObject);
begin
  ShowModemLog;
  try
    SendSmsPhn(DA.Text, Message.Text);
  finally
    HideModemLog;
  end;
end;

procedure TGSMCentralForm.ShowModemLog;
begin
  if AutoShowModemLog.Checked then
  begin
    FSaveActivePage:= PageControl1.ActivePage;
    PageControl1.ActivePage:= TabSheet7;
    Application.ProcessMessages;
  end;
end;

procedure TGSMCentralForm.HideModemLog;
begin
  if AutoShowModemLog.Checked then
  begin
    PageControl1.ActivePage:= FSaveActivePage;
  end;
end;

procedure TGSMCentralForm.GetConfiguration;
var i: Integer;
begin
  with GSMDataModule,GSM do
  begin
    ModemComboBox.ItemIndex := Ord(Equipment);
    SCAEdit.Text := SCA;
    PINEdit.Text := PIN;
    Self.SMSFormat.ItemIndex := SMSFormat;
    CatchSMSMT.Checked := uindSMSDeliver in UnsolicitedIndication;
    CatchSMSCB.Checked := uindCellBroadcast in UnsolicitedIndication;
    CatchSMSDS.Checked := uindSMSStatusReport in UnsolicitedIndication;
    OnlyIndication.Checked := uindOnlyIndication in UnsolicitedIndication;
    with CommRadioGroup do
      for i:=0 to Items.Count-1 do
        if ComDevice.DeviceName=Items[i] then
        begin
          ItemIndex := i;
          break;
        end;

    SMSLogCheckBox.Checked := SMSLogFile <> NO_LOG_FILE;
    GSMLogCheckBox.Checked := GSMLogFile <> NO_LOG_FILE;
    SMSLogSaveDialog.FileName := SMSLogFile;
    GSMLogSaveDialog.FileName := GSMLogFile;
  end;
end;

procedure TGSMCentralForm.SetConfiguration;
begin
  with GSMDataModule,GSM do
  begin
    Equipment := TGSMEquipment(ModemComboBox.ItemIndex);
    SCA := SCAEdit.Text;
    PIN := PINEdit.Text;
    SMSFormat:= Self.SMSFormat.ItemIndex;
    UnsolicitedIndication:= [];
    if CatchSMSMT.Checked then
      UnsolicitedIndication:= UnsolicitedIndication+[uindSMSDeliver];
    if CatchSMSCB.Checked then
      UnsolicitedIndication:= UnsolicitedIndication+[uindCellBroadcast];
    if CatchSMSDS.Checked then
      UnsolicitedIndication:= UnsolicitedIndication+[uindSMSStatusReport];
    if OnlyIndication.Checked then
      UnsolicitedIndication:= UnsolicitedIndication+[uindOnlyIndication];

    ComDevice.DeviceName := CommRadioGroup.Items[CommRadioGroup.ItemIndex];

    if SMSLogCheckBox.Checked then
      SMSLogFile := SMSLogSaveDialog.FileName
    else
      SMSLogFile := NO_LOG_FILE;

    if GSMLogCheckBox.Checked then
      GSMLogFile := GSMLogSaveDialog.FileName
    else
      GSMLogFile := NO_LOG_FILE;

    self.Caption:= Format(CaptionStr, [GSMEquipmentLongName[Equipment]]);
  end;

end;

procedure TGSMCentralForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  SetConfiguration;
end;

procedure TGSMCentralForm.SMSLOGCheckBoxClick(Sender: TObject);
begin
  SMSLogButton.Enabled := SMSLogCheckBox.Checked
end;

procedure TGSMCentralForm.GSMLOGCheckBoxClick(Sender: TObject);
begin
  GSMLogButton.Enabled := GSMLogCheckBox.Checked
end;

procedure TGSMCentralForm.SMSLogButtonClick(Sender: TObject);
begin
  if SMSLogCheckBox.Checked then
  with SMSLogSaveDialog,GSMDataModule  do
    if Execute
    then
      SMSLogFile := FileName
    else
      FileName := SMSLogFile;
end;

procedure TGSMCentralForm.GSMLogButtonClick(Sender: TObject);
begin
  if GSMLogCheckBox.Checked then
  with GSMLogSaveDialog,GSMDataModule  do
    if Execute
    then
      GSMLogFile := FileName
    else
      FileName := GSMLogFile;
end;

end.

